/*
* Copyright © 2010 Red Hat, Inc.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*           http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

package com.redhat.rhevm.api.powershell.model;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import javax.xml.datatype.XMLGregorianCalendar;

import com.redhat.rhevm.api.common.util.SizeConverter;
import com.redhat.rhevm.api.common.util.StatusUtils;
import com.redhat.rhevm.api.model.Disk;
import com.redhat.rhevm.api.model.StorageDomain;
import com.redhat.rhevm.api.model.StorageDomains;
import com.redhat.rhevm.api.model.VM;
import com.redhat.rhevm.api.powershell.enums.PowerShellDiskInterface;
import com.redhat.rhevm.api.powershell.enums.PowerShellDiskType;
import com.redhat.rhevm.api.powershell.enums.PowerShellImageStatus;
import com.redhat.rhevm.api.powershell.enums.PowerShellPropagateErrors;
import com.redhat.rhevm.api.powershell.enums.PowerShellVolumeFormat;
import com.redhat.rhevm.api.powershell.enums.PowerShellVolumeType;
import com.redhat.rhevm.api.powershell.util.PowerShellParser;
import com.redhat.rhevm.api.powershell.util.PowerShellUtils;

import static com.redhat.rhevm.api.powershell.util.PowerShellUtils.last;

public class PowerShellDisk extends Disk {

    private static final String DISKS_TYPE = "RhevmCmd.CLIImage";

    private String vmSnapshotId;
    private String parentId;
    private XMLGregorianCalendar lastModified;
    private String internalDriveMapping;
    private String taskIds;

    static boolean isDisk(PowerShellParser.Entity entity) {
        return DISKS_TYPE.equals(entity.getType());
    }

    public String getVmSnapshotId() {
        return vmSnapshotId;
    }
    public void setVmSnapshotId(String vmSnapshotId) {
        this.vmSnapshotId = vmSnapshotId;
    }

    public String getParentId() {
        return parentId;
    }
    public void setParentId(String parentId) {
        this.parentId = parentId;
    }

    public XMLGregorianCalendar getLastModified() {
        return lastModified;
    }
    public void setLastModified(XMLGregorianCalendar lastModified) {
        this.lastModified = lastModified;
    }

    public String getInternalDriveMapping() {
        return internalDriveMapping;
    }
    public void setInternalDriveMapping(String internalDriveMapping) {
        this.internalDriveMapping = internalDriveMapping;
    }

    public String getTaskIds() {
        return taskIds;
    }

    public void setTaskIds(String taskIds) {
        this.taskIds = taskIds;
    }

    public static List<PowerShellDisk> parse(PowerShellParser parser, String vmId, String output) {
        List<PowerShellDisk> ret = new ArrayList<PowerShellDisk>();

        Map<String, XMLGregorianCalendar> dates = new HashMap<String, XMLGregorianCalendar>();
        String date = null;

        String storageDomainId = null;

        for (PowerShellParser.Entity entity : parser.parse(output)) {
            if (PowerShellParser.DATE_TYPE.equals(entity.getType())) {
                date = entity.getValue();
            } else if (PowerShellParser.STRING_TYPE.equals(entity.getType())) {
                dates.put(date, PowerShellUtils.parseDate(entity.getValue()));
                date = null;
            } else if (PowerShellStorageDomain.isStorageDomain(entity)) {
                storageDomainId = PowerShellStorageDomain.parseEntity(entity).getId();
            } else if (PowerShellAsyncTask.isTask(entity)) {
                last(ret).setTaskIds(PowerShellAsyncTask.parseTask(entity, last(ret).getTaskIds()));
            } else if (PowerShellAsyncTask.isStatus(entity)) {
                String creationStatus = last(ret).getCreationStatus()!=null ? last(ret).getCreationStatus().getState() : null;
                last(ret).setCreationStatus(StatusUtils.create(PowerShellAsyncTask.parseStatus(entity, StatusUtils.getRequestStatus(creationStatus))));
            } else if (isDisk(entity)) {
                ret.add(parseEntity(vmId, entity, storageDomainId, dates));
            }
        }

        return ret;
    }

    public static PowerShellDisk parseEntity(String vmId, PowerShellParser.Entity entity, String storageDomainId){
        return parseEntity(vmId, entity, storageDomainId, null);
    }

    private static PowerShellDisk parseEntity(String vmId,
                                              PowerShellParser.Entity entity,
                                              String storageDomainId,
                                              Map<String, XMLGregorianCalendar> dates) {
        PowerShellDisk disk = new PowerShellDisk();

        disk.setId(entity.get("snapshotid"));
        disk.setDescription(entity.get("description"));

        disk.setVm(new VM());
        disk.getVm().setId(vmId);

        if (storageDomainId != null) {
            disk.setStorageDomains(new StorageDomains());
            StorageDomain storageDomain = new StorageDomain();
            storageDomain.setId(storageDomainId);
            disk.getStorageDomains().getStorageDomains().add(storageDomain);
        }

        disk.setSize(SizeConverter.gigasToBytes(entity.get("sizeingb", Long.class)));
        disk.setType(entity.get("disktype", PowerShellDiskType.class).map().value());
        disk.setStatus(StatusUtils.create(entity.get("status", PowerShellImageStatus.class).map().value()));
        disk.setInterface(PowerShellDiskInterface.valueOf(entity.get("diskinterface")).map().value());
        disk.setFormat(entity.get("volumeformat", PowerShellVolumeFormat.class).map().value());
        disk.setSparse(entity.get("volumetype", PowerShellVolumeType.class).map());
        disk.setBootable(entity.get("boot", Boolean.class));
        disk.setWipeAfterDelete(entity.get("wipeafterdelete", Boolean.class));
        disk.setPropagateErrors(entity.get("propagateerrors", PowerShellPropagateErrors.class).map());

        disk.setVmSnapshotId(entity.get("vmsnapshotid"));
        disk.setParentId(entity.get("parentid"));
        disk.setInternalDriveMapping(entity.get("internaldrivemapping"));

        if (dates != null && dates.containsKey(entity.get("lastmodified"))) {
            disk.setLastModified(dates.get(entity.get("lastmodified")));
        }

        return disk;
    }
}
