/*
* Copyright © 2010 Red Hat, Inc.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*           http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

package com.redhat.rhevm.api.powershell.resource;

import java.util.List;
import java.util.concurrent.Executor;

import com.redhat.rhevm.api.model.IP;
import com.redhat.rhevm.api.model.Network;
import com.redhat.rhevm.api.resource.NetworkResource;
import com.redhat.rhevm.api.common.resource.UriInfoProvider;
import com.redhat.rhevm.api.common.util.LinkHelper;
import com.redhat.rhevm.api.powershell.model.PowerShellNetwork;
import com.redhat.rhevm.api.powershell.util.PowerShellCmd;
import com.redhat.rhevm.api.powershell.util.PowerShellParser;
import com.redhat.rhevm.api.powershell.util.PowerShellPool;
import com.redhat.rhevm.api.powershell.util.PowerShellPoolMap;
import com.redhat.rhevm.api.powershell.util.PowerShellUtils;

public class PowerShellNetworkResource extends AbstractPowerShellActionableResource<Network> implements NetworkResource {

    public PowerShellNetworkResource(String id,
                                     Executor executor,
                                     UriInfoProvider uriProvider,
                                     PowerShellPoolMap shellPools,
                                     PowerShellParser parser) {
        super(id, executor, uriProvider, shellPools, parser);
    }

    public static List<Network> runAndParse(PowerShellPool pool, PowerShellParser parser, String command) {
        return PowerShellNetwork.parse(parser, PowerShellCmd.runCommand(pool, command));
    }

    public static Network runAndParseSingle(PowerShellPool pool, PowerShellParser parser, String command) {
        List<Network> networks = runAndParse(pool, parser, command);

        return !networks.isEmpty() ? networks.get(0) : null;
    }

    public List<Network> runAndParse(String command) {
        return runAndParse(getPool(), getParser(), command);
    }

    public Network runAndParseSingle(String command) {
        return runAndParseSingle(getPool(), getParser(), command);
    }

    @Override
    public Network get() {
        StringBuilder buf = new StringBuilder();

        buf.append("$n = get-networks;");
        buf.append("foreach ($i in $n) {");
        buf.append("  if ($i.networkid -eq " + PowerShellUtils.escape(getId()) + ") {");
        buf.append("    $i");
        buf.append("  }");
        buf.append("}");

        return LinkHelper.addLinks(getUriInfo(), runAndParseSingle(buf.toString()));
    }

    @Override
    public Network update(Network network) {
        validateUpdate(network);

        StringBuilder buf = new StringBuilder();

        buf.append("$n = get-networks;");
        buf.append("foreach ($i in $n) { ");
        buf.append("if ($i.networkid -eq " + PowerShellUtils.escape(getId()) + ") { ");

        if (network.getName() != null) {
            buf.append("$i.name = " + PowerShellUtils.escape(network.getName()) + "; ");
        }
        if (network.getDescription() != null) {
            buf.append("$i.description = " + PowerShellUtils.escape(network.getDescription()) + "; ");
        }

        if (network.getIp() != null) {
            IP ip = network.getIp();

            if (ip.getAddress() != null) {
                buf.append("$i.address = " + PowerShellUtils.escape(ip.getAddress()) + "; ");
            }
            if (ip.getNetmask() != null) {
                buf.append("$i.subnet = " + PowerShellUtils.escape(ip.getNetmask()) + "; ");
            }
            if (ip.getGateway() != null) {
                buf.append("$i.gateway = " + PowerShellUtils.escape(ip.getGateway()) + "; ");
            }
        }

        if (network.getVlan() != null) {
            buf.append("$i.vlanid = " + Integer.toString(network.getVlan().getId()) + "; ");
        }
        if (network.isStp() != null) {
            if (network.isStp()) {
                buf.append("$i.stp = $true; ");
            } else {
                buf.append("$i.stp = $false; ");
            }
        }
        if (network.isDisplay() != null) {
            if (network.isDisplay()) {
                buf.append("$i.isdisplay = $true; ");
            } else {
                buf.append("$i.isdisplay = $false; ");
            }
        }

        buf.append("update-network");
        buf.append(" -networkobject $i");
        buf.append(" -datacenterid $i.datacenterid");

        buf.append(" } ");
        buf.append("}");

        return LinkHelper.addLinks(getUriInfo(), runAndParseSingle(buf.toString()));
    }
}
